package cn.minbb.edu.controller.rest;

import cn.minbb.edu.data.ResponseResult;
import cn.minbb.edu.model.Course;
import cn.minbb.edu.model.User;
import cn.minbb.edu.service.CourseService;
import cn.minbb.edu.service.SubjectService;
import cn.minbb.edu.service.UserService;
import cn.minbb.edu.util.CollectionUtil;
import com.alibaba.fastjson.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("app/course")
public class CourseController {

    private UserService userService;
    private CourseService courseService;
    private SubjectService subjectService;

    @Autowired
    public CourseController(UserService userService, CourseService courseService, SubjectService subjectService) {
        this.userService = userService;
        this.courseService = courseService;
        this.subjectService = subjectService;
    }

    @GetMapping("")
    public ResponseResult<Course> course(@RequestParam("id") Integer id) {
        Course course = courseService.findOneById(id);
        course.setVoTeacher(userService.findUserById(course.getTeacherId()));
        return ResponseResult.ok(true).code(1).data(course);
    }

    @PostMapping("hot")
    public ResponseResult<Course> courseHot() {
        List<Course> courseList = courseService.findTop6Hot();
        int count = courseService.countAllHot();
        return ResponseResult.ok(true).code(1).message(String.format("%s", count)).dataset(courseList);
    }

    @PostMapping("excellent")
    public ResponseResult<Course> courseExcellent() {
        List<Course> courseList = courseService.findTop6Excellent();
        int count = courseService.countAllExcellent();
        return ResponseResult.ok(true).code(1).message(String.format("%s", count)).dataset(courseList);
    }

    @PostMapping("all")
    public ResponseResult<Course> courseAll() {
        return ResponseResult.ok(true).code(1).dataset(courseService.findAll());
    }

    @PostMapping("user/all")
    public ResponseResult<Course> userAll(@RequestBody String userData) {
        User userClient = JSONObject.parseObject(userData, User.class);
        User user = userService.findUserById(userClient.getId());
        List<Course> courseList = courseService.findAllByIdIn(CollectionUtil.stringToIntList(user.getCourseIds()));
        return ResponseResult.ok(true).code(1).message("查询到 " + courseList.size() + "门课程").dataset(courseList);
    }

    @PostMapping("user/add")
    public ResponseResult<Course> userAdd(@RequestBody Map<String, Object> dataMap) {
        User userClient = JSONObject.parseObject(JSONObject.toJSONString(dataMap.get("user")), User.class);
        User user = userService.findUserById(userClient.getId());
        Course course = JSONObject.parseObject(JSONObject.toJSONString(dataMap.get("course")), Course.class);
        List<Integer> courseIdList = CollectionUtil.stringToIntList(user.getCourseIds());
        if (!courseIdList.contains(course.getId())) courseIdList.add(course.getId());
        user.setCourseIds(CollectionUtil.intListToString(courseIdList));
        userService.save(user);
        return ResponseResult.ok(true).code(1).message("加入我的课程成功").dataNone();
    }

    @PostMapping("user/remove")
    public ResponseResult<Course> userRemove(@RequestBody Map<String, Object> dataMap) {
        User userClient = JSONObject.parseObject(JSONObject.toJSONString(dataMap.get("user")), User.class);
        User user = userService.findUserById(userClient.getId());
        Course course = JSONObject.parseObject(JSONObject.toJSONString(dataMap.get("course")), Course.class);
        List<Integer> courseIdList = CollectionUtil.stringToIntList(user.getCourseIds());
        if (courseIdList.contains(course.getId())) courseIdList.remove(course.getId());
        user.setCourseIds(CollectionUtil.intListToString(courseIdList));
        userService.save(user);
        return ResponseResult.ok(true).code(1).message("移除成功").dataNone();
    }
}
