package cn.minbb.edu.controller.web;

import cn.minbb.edu.model.Course;
import cn.minbb.edu.model.Subject;
import cn.minbb.edu.model.User;
import cn.minbb.edu.service.CourseService;
import cn.minbb.edu.service.SubjectService;
import cn.minbb.edu.service.UserService;
import cn.minbb.edu.storage.FileService;
import cn.minbb.edu.storage.StorageProperties;
import cn.minbb.edu.system.Const;
import cn.minbb.edu.system.UserSession;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

@Controller
@RequestMapping("course")
public class CourseController {

    private UserService userService;
    private CourseService courseService;
    private SubjectService subjectService;
    private FileService fileService;

    @Autowired
    public CourseController(UserService userService, CourseService courseService, SubjectService subjectService, FileService fileService) {
        this.userService = userService;
        this.courseService = courseService;
        this.subjectService = subjectService;
        this.fileService = fileService;
    }

    @GetMapping(value = "create")
    public ModelAndView courseCreatePage(ModelAndView modelAndView, HttpServletRequest request) {
        modelAndView.addObject("subjectList", subjectService.findAll());
        modelAndView.setViewName("course-create");
        return modelAndView;
    }

    @PostMapping(value = "create")
    public ModelAndView courseCreate(
            @RequestParam("name") String name,
            @RequestParam("subject") String subjectName,
            @RequestParam("introduction") String introduction,
            @RequestParam("remark") String remark,
            @RequestParam("cover") MultipartFile cover,
            @RequestParam("video") MultipartFile video,
            ModelAndView modelAndView, HttpServletRequest request) {
        User user = UserSession.getUserAuthentication();
        if (null != user) {
            int userId = user.getId();
            String coverFilename = cover.getOriginalFilename();
            String videoFilename = video.getOriginalFilename();
            String filenamePrefix = userId + "-" + System.currentTimeMillis();
            if (null != coverFilename && null != videoFilename) {
                String coverFilenameStore = filenamePrefix + coverFilename.substring(coverFilename.lastIndexOf("."));
                String videoFilenameStore = filenamePrefix + videoFilename.substring(videoFilename.lastIndexOf("."));
                fileService.storeToFolder(cover, coverFilenameStore, StorageProperties.Folder.IMAGES);
                fileService.storeToFolder(video, videoFilenameStore, StorageProperties.Folder.VIDEOS);
                // 查询学科
                Subject subject = subjectService.findOneByName(subjectName);
                Course course = new Course();
                course.setName(name);
                course.setIntroduction(introduction);
                course.setRemark(remark);
                course.setCover(Const.STORAGE_HOST + "images/" + coverFilenameStore);
                course.setVideo(Const.STORAGE_HOST + "videos/" + videoFilenameStore);
                course.setTeacherId(userId);
                if (null != subject) course.setSubjectId(subject.getId());
                else course.setSubjectId(0);
                courseService.save(course);
            }
        }
        modelAndView.setViewName("redirect:/course/center");
        return modelAndView;
    }

    @GetMapping(value = "square")
    public ModelAndView courseSquarePage(ModelAndView modelAndView, HttpServletRequest request) {
        modelAndView.setViewName("course-square");
        return modelAndView;
    }

    @GetMapping(value = "center")
    public ModelAndView courseCenterPage(ModelAndView modelAndView, HttpServletRequest request) {
        User user = UserSession.getUserAuthentication();
        if (null != user) {
            List<Course> courseList = courseService.findAllByUserId(user.getId());
            modelAndView.addObject("courseList", courseList);
            modelAndView.setViewName("course-center");
        } else {
            modelAndView.setViewName("redirect:/");
        }
        return modelAndView;
    }

    @GetMapping(value = "player")
    public ModelAndView coursePlayerPage(
            @RequestParam(value = "id", required = false) Integer id,
            ModelAndView modelAndView, HttpServletRequest request) {
        if (null != id) {
            Course course = courseService.findOneById(id);
            modelAndView.addObject("course", course);
        }
        modelAndView.setViewName("course-player");
        return modelAndView;
    }
}
