package cn.minbb.edu.controller.web;

import cn.minbb.edu.storage.FileService;
import cn.minbb.edu.storage.StorageFileNotFoundException;
import cn.minbb.edu.storage.StorageProperties;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;

@Controller
@RequestMapping("file")
public class FileController {

    private FileService fileService;

    public FileController(FileService fileService) {
        this.fileService = fileService;
    }

    @GetMapping(value = "/images/{filename:.+}")
    public ResponseEntity<Resource> loadImages(@PathVariable String filename) {
        Resource file = fileService.loadAsResourceFromFolder(filename, StorageProperties.Folder.IMAGES);
        return ResponseEntity.ok().header(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"" + file.getFilename() + "\"").body(file);
    }

    @GetMapping(value = "/videos/{filename:.+}")
    public ResponseEntity<Resource> loadVideos(@PathVariable String filename) {
        Resource file = fileService.loadAsResourceFromFolder(filename, StorageProperties.Folder.VIDEOS);
        return ResponseEntity.ok().header(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"" + file.getFilename() + "\"").body(file);
    }

    @GetMapping(value = "/download/videos/{filename:.+}")
    public ResponseEntity<Resource> downloadVideos(@PathVariable String filename) {
        Resource file = fileService.loadAsResourceFromFolder(filename, StorageProperties.Folder.VIDEOS);
        return ResponseEntity.ok().header(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"" + file.getFilename() + "\"").body(file);
    }

    @ExceptionHandler(value = StorageFileNotFoundException.class)
    public ResponseEntity handleStorageFileNotFound(StorageFileNotFoundException e) {
        return ResponseEntity.notFound().build();
    }
}
