package cn.minbb.edu.controller.web;

import cn.minbb.edu.model.Banner;
import cn.minbb.edu.service.BannerService;
import cn.minbb.edu.service.UserService;
import cn.minbb.edu.system.Const;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.List;

@Controller
@RequestMapping("")
public class MainController {

    private UserService userService;
    private BannerService bannerService;

    @Autowired
    public MainController(UserService userService, BannerService bannerService) {
        this.userService = userService;
        this.bannerService = bannerService;
    }

    @GetMapping(value = "")
    public ModelAndView indexPage(ModelAndView modelAndView, HttpServletRequest request) {
        List<Banner> bannerList = bannerService.findAll();
        modelAndView.addObject("bannerList", bannerList);
        modelAndView.addObject(Const.ACTIVE, "index");
        modelAndView.setViewName("index");
        return modelAndView;
    }

    @GetMapping(value = "about")
    public ModelAndView aboutPage(ModelAndView modelAndView, HttpServletRequest request) {
        modelAndView.addObject(Const.ACTIVE, "about");
        modelAndView.setViewName("index-about");
        return modelAndView;
    }

    @GetMapping(value = "download")
    public ResponseEntity<InputStreamResource> downloadPage(HttpServletRequest request) throws IOException {
        // String apkPath = "C:\\Evaluation\\download\\apk\\";
        // FileSystemResource file = new FileSystemResource(apkPath);
        ClassPathResource file = new ClassPathResource("/static/favicon.ico", this.getClass());
        HttpHeaders headers = new HttpHeaders();
        headers.add("Cache-Control", "no-cache, no-store, must-revalidate");
        headers.add("Content-Disposition", String.format("attachment; filename=\"%s\"", file.getFilename()));
        headers.add("Pragma", "no-cache");
        headers.add("Expires", "0");
        return ResponseEntity.ok().headers(headers).contentLength(file.contentLength())
                .contentType(MediaType.parseMediaType("application/octet-stream"))
                .contentType(MediaType.parseMediaType("application/x-shockwave-flash"))
                .body(new InputStreamResource(file.getInputStream()));
    }

    @GetMapping(value = {"register", "sign-up"})
    public ModelAndView signUp(ModelAndView modelAndView, HttpServletRequest request) {
        modelAndView.setViewName("sign-up");
        return modelAndView;
    }

    @GetMapping(value = {"login", "sign-in"})
    public ModelAndView signIn(ModelAndView modelAndView, HttpServletRequest request) {
        modelAndView.setViewName("sign-in");
        return modelAndView;
    }
}
