package cn.minbb.edu.data;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serializable;
import java.util.Collection;

@Data
@NoArgsConstructor
@AllArgsConstructor
public class ResponseResult<T extends Serializable> implements Serializable {

    private Integer code = -1;

    private Boolean success = false;

    private String message = "";

    private T data;

    private Collection<T> dataset;

    public static ResponseResult.Builder ok(Boolean success) {
        return new ResultBuilder(success);
    }

    private static class ResultBuilder implements Builder {

        private Integer code;
        private Boolean success;
        private String message;

        ResultBuilder(boolean success) {
            this.success = success;
        }

        @Override
        public Builder code(Integer code) {
            this.code = code;
            return this;
        }

        @Override
        public Builder message(String message) {
            this.message = message;
            return this;
        }

        @Override
        public <T extends Serializable> ResponseResult<T> data(T data) {
            return new ResponseResult<>(code, success, message, data, null);
        }

        @Override
        public <T extends Serializable> ResponseResult<T> dataset(Collection<T> dataset) {
            return new ResponseResult<>(code, success, message, null, dataset);
        }

        @Override
        public <T extends Serializable> ResponseResult<T> dataAll(T data, Collection<T> dataset) {
            return new ResponseResult<>(code, success, message, data, dataset);
        }

        @Override
        public <T extends Serializable> ResponseResult<T> dataNone() {
            return new ResponseResult<>(code, success, message, null, null);
        }
    }

    public interface Builder {
        Builder code(Integer code);

        Builder message(String message);

        <T extends Serializable> ResponseResult<T> data(T data);

        <T extends Serializable> ResponseResult<T> dataset(Collection<T> dataset);

        <T extends Serializable> ResponseResult<T> dataAll(T data, Collection<T> dataset);

        <T extends Serializable> ResponseResult<T> dataNone();
    }
}
