package cn.minbb.edu.handler;

import cn.minbb.edu.data.ResponseResult;
import cn.minbb.edu.util.RequestUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.WebAttributes;
import org.springframework.security.web.authentication.SimpleUrlAuthenticationFailureHandler;
import org.springframework.stereotype.Component;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

@Component
public class LoginAuthenticationFailureHandler extends SimpleUrlAuthenticationFailureHandler {

    private Logger logger = LoggerFactory.getLogger(LoginAuthenticationFailureHandler.class);

    private ObjectMapper objectMapper;

    @Autowired
    public LoginAuthenticationFailureHandler(ObjectMapper objectMapper) {
        this.objectMapper = objectMapper;
    }

    @Override
    public void onAuthenticationFailure(HttpServletRequest request, HttpServletResponse response, AuthenticationException exception) throws IOException, ServletException {
        String message = objectMapper.writeValueAsString(exception.getMessage());
        logger.info("用户登录失败 = {}", message);
        if (RequestUtil.isAjax(request)) {
            response.setStatus(HttpStatus.INTERNAL_SERVER_ERROR.value());
            response.setContentType("application/json;charset=UTF-8");
            response.getWriter().write(objectMapper.writeValueAsString(
                    ResponseResult.ok(false).code(-1).message(exception.getMessage()).dataNone()));
        } else {
            // 异常写入 Session
            request.getSession().setAttribute(WebAttributes.AUTHENTICATION_EXCEPTION, exception);
            response.setStatus(HttpStatus.OK.value());
            response.sendRedirect("/sign-in?error=true");
        }
    }
}
