package cn.minbb.edu.handler;

import cn.minbb.edu.util.RequestUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.core.Authentication;
import org.springframework.security.web.authentication.SavedRequestAwareAuthenticationSuccessHandler;
import org.springframework.security.web.savedrequest.SavedRequest;
import org.springframework.stereotype.Component;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

@Component
public class LoginAuthenticationSuccessHandler extends SavedRequestAwareAuthenticationSuccessHandler {

    private Logger logger = LoggerFactory.getLogger(LoginAuthenticationSuccessHandler.class);

    private ObjectMapper objectMapper;

    @Autowired
    public LoginAuthenticationSuccessHandler(ObjectMapper objectMapper) {
        this.objectMapper = objectMapper;
    }

    // Authentication 封装认证信息
    @Override
    public void onAuthenticationSuccess(HttpServletRequest request, HttpServletResponse response, Authentication authentication) throws ServletException, IOException {
        String auth = objectMapper.writeValueAsString(authentication);
        logger.info("用户登陆成功 = {}", auth);
        // 登录方式不同，Authentication 不同
        if (RequestUtil.isAjax(request)) {
            response.setContentType("application/json;charset=UTF-8");
            // 把 authentication 对象转成 json 格式字符串通过 response 以 application/json;charset=UTF-8 格式写到响应里面去
            response.getWriter().write(objectMapper.writeValueAsString(authentication));
        } else {
            // 跳转到登录之前访问的页面
            response.setStatus(HttpStatus.OK.value());
            SavedRequest savedRequest = (SavedRequest) request.getSession().getAttribute("SPRING_SECURITY_SAVED_REQUEST");
            if (savedRequest != null) {
                response.sendRedirect(savedRequest.getRedirectUrl());
                request.getSession().removeAttribute("SPRING_SECURITY_SAVED_REQUEST");
            } else {
                // 重定向页面需要在登录页设置路径参数
                String redirect = (String) request.getSession().getAttribute("REDIRECT");
                if (null != redirect) {
                    response.sendRedirect(redirect);
                } else {
                    response.sendRedirect("/");
                }
            }
        }
    }
}
