package cn.minbb.edu.handler;

import cn.minbb.edu.util.RequestUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.core.Authentication;
import org.springframework.security.web.authentication.logout.SimpleUrlLogoutSuccessHandler;
import org.springframework.stereotype.Component;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * SimpleUrlLogoutSuccessHandler - 重定向到设置的URL地址。默认的地址是/login?logout
 * HttpStatusReturningLogoutSuccessHandler - REST API场景，允许设置一个返回给客户端的HTTP状态码（默认返回200）来替换重定向到URL这个动作
 */
@Component
public class LogoutSuccessHandler extends SimpleUrlLogoutSuccessHandler {

    private Logger logger = LoggerFactory.getLogger(LogoutSuccessHandler.class);

    private ObjectMapper objectMapper;

    @Autowired
    public LogoutSuccessHandler(  ObjectMapper objectMapper) {
        this.objectMapper = objectMapper;
    }

    @Override
    public void onLogoutSuccess(HttpServletRequest request, HttpServletResponse response, Authentication authentication) throws IOException, ServletException {
        String auth = objectMapper.writeValueAsString(authentication);
        logger.info("用户登出成功 = {}", auth);
        if (RequestUtil.isAjax(request)) {
            response.setContentType("application/json;charset=UTF-8");
            response.getWriter().write(objectMapper.writeValueAsString(authentication));
        } else {
            response.setStatus(HttpStatus.OK.value());
            response.sendRedirect("/sign-in?logout=true");
            super.onLogoutSuccess(request, response, authentication);
        }
    }
}
