package cn.minbb.edu.model;

import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import org.hibernate.annotations.CreationTimestamp;
import org.hibernate.annotations.UpdateTimestamp;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Date;

@Entity
@Table(name = "course")
@NoArgsConstructor
@AllArgsConstructor
public class Course implements Serializable {

    @Getter
    @Setter
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id", nullable = false, columnDefinition = "INT COMMENT '课程实体'")
    private Integer id;

    @Getter
    @Setter
    @Column(name = "name", nullable = false, columnDefinition = "VARCHAR(64) COMMENT '名称'")
    private String name;

    @Getter
    @Setter
    @Column(name = "introduction", nullable = false, columnDefinition = "VARCHAR(255) COMMENT '介绍'")
    private String introduction;

    @Getter
    @Setter
    @Column(name = "cover", columnDefinition = "VARCHAR(255) COMMENT '封面'")
    private String cover;

    @Getter
    @Setter
    @Column(name = "video", columnDefinition = "VARCHAR(255) COMMENT '视频'")
    private String video;

    @Getter
    @Setter
    @Column(name = "remark", columnDefinition = "VARCHAR(255) COMMENT '备注'")
    private String remark;

    @Getter
    @Setter
    @Column(name = "count_hot", nullable = false, columnDefinition = "INT COMMENT '热门'")
    private Integer countHot;

    @Getter
    @Setter
    @Column(name = "count_excellent", nullable = false, columnDefinition = "INT COMMENT '优秀'")
    private Integer countExcellent;

    @Getter
    @Setter
    @Column(name = "subject_id", nullable = false, columnDefinition = "INT COMMENT '关联学科'")
    private Integer subjectId;

    @Getter
    @Setter
    @Column(name = "teacher_id", nullable = false, columnDefinition = "INT COMMENT '关联用户（教师）'")
    private Integer teacherId;

    @Getter
    @Setter
    @Column(name = "is_enabled", nullable = false, columnDefinition = "TINYINT DEFAULT '1' COMMENT '已启用'")
    private Boolean isEnabled;

    @Getter
    @Setter
    @Column(name = "created_at", columnDefinition = "DATETIME COMMENT '创建时间'")
    @CreationTimestamp
    private Date createdAt;

    @Getter
    @Setter
    @Column(name = "updated_at", columnDefinition = "DATETIME COMMENT '更新时间'")
    @UpdateTimestamp
    private Date updatedAt;

    @Version
    @Column(name = "version", columnDefinition = "INTEGER COMMENT '版本号'")
    public Integer version;

    @Getter
    @Setter
    @Transient
    private User voTeacher;

    public Course(Boolean isEnabled) {
        this.isEnabled = isEnabled;
    }
}
