package cn.minbb.edu.model;

import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

import javax.persistence.*;
import java.io.Serializable;

@Entity
@Table(name = "user_role",
        uniqueConstraints = {@UniqueConstraint(name = "unique_role", columnNames = {"role"})},
        indexes = {@Index(name = "index_role", columnList = "role")})
@NoArgsConstructor
public class UserRole implements Serializable {

    @Getter
    @Setter
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id", nullable = false, columnDefinition = "INT COMMENT '用户角色'")
    private Integer id;

    @Getter
    @Setter
    @Enumerated(EnumType.STRING)
    @Column(name = "role", nullable = false, columnDefinition = "VARCHAR(32) COMMENT '角色名称'")
    private Role role;

    @Getter
    @Setter
    @Column(name = "description", columnDefinition = "VARCHAR(255) COMMENT '角色描述'")
    private String description;

    public UserRole(Role role) {
        this.role = role;
    }

    public UserRole(Role role, String description) {
        this.role = role;
        this.description = description;
    }

    @Getter
    public enum Role {
        USER("用户"),
        STUDENT("学生"),
        TEACHER("教师"),
        ADMIN("管理员");

        String description;

        Role(String description) {
            this.description = description;
        }
    }
}
