package cn.minbb.edu.service.impl;

import cn.minbb.edu.model.Course;
import cn.minbb.edu.model.repository.CourseRepository;
import cn.minbb.edu.service.CourseService;
import cn.minbb.edu.util.SortTools;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class CourseServiceImpl implements CourseService {

    private CourseRepository courseRepository;

    @Autowired
    public CourseServiceImpl(CourseRepository courseRepository) {
        this.courseRepository = courseRepository;
    }

    @Override
    public Course save(Course course) {
        course.setCountHot(0);
        course.setCountExcellent(0);
        course.setIsEnabled(true);
        return courseRepository.save(course);
    }

    @Override
    public Course findOneById(Integer id) {
        Course course = new Course(true);
        course.setId(id);
        Example<Course> example = Example.of(course);
        return courseRepository.findOne(example).orElse(null);
    }

    @Override
    public List<Course> findAll() {
        return courseRepository.findAll(SortTools.createdAtDown());
    }

    @Override
    public List<Course> findAllByUserId(Integer userId) {
        Course course = new Course(true);
        course.setTeacherId(userId);
        return courseRepository.findAll(Example.of(course), SortTools.createdAtDown());
    }

    @Override
    public List<Course> findAllBySubjectId(Integer subjectId) {
        Course course = new Course(true);
        course.setSubjectId(subjectId);
        return courseRepository.findAll(Example.of(course), SortTools.createdAtDown());
    }

    @Override
    public List<Course> findAllByIdIn(List<Integer> ids) {
        return courseRepository.findAllByIdIn(ids);
    }

    @Override
    public List<Course> findTop6Hot() {
        return courseRepository.findTop6ByIsEnabledTrue(SortTools.basicDescSort("countHot"));
    }

    @Override
    public List<Course> findTop6Excellent() {
        return courseRepository.findTop6ByIsEnabledTrue(SortTools.basicDescSort("countExcellent"));
    }

    @Override
    public int countAllHot() {
        return courseRepository.countByIsEnabledTrue(SortTools.basicDescSort("countHot"));
    }

    @Override
    public int countAllExcellent() {
        return courseRepository.countByIsEnabledTrue(SortTools.basicDescSort("countExcellent"));
    }
}
