package cn.minbb.edu.service.impl;

import cn.minbb.edu.model.User;
import cn.minbb.edu.model.UserRole;
import cn.minbb.edu.model.repository.UserRepository;
import cn.minbb.edu.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.CredentialsExpiredException;
import org.springframework.security.authentication.DisabledException;
import org.springframework.security.authentication.LockedException;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

@Service
public class UserServiceImpl implements UserService {

    private UserRepository userRepository;

    @Autowired
    public UserServiceImpl(UserRepository userRepository) {
        this.userRepository = userRepository;
    }

    @Override
    public User save(User user) {
        user.setIsAccountNonExpired(true);
        user.setIsAccountNonLocked(true);
        user.setIsCredentialsNonExpired(true);
        user.setIsEnabled(true);
        return userRepository.save(user);
    }

    @Override
    public User findUserById(Integer id) {
        User user = new User();
        user.setId(id);
        return null == id ? null : userRepository.findOne(Example.of(user)).orElse(null);
    }

    @Override
    public User findUserByUsername(String username) {
        return userRepository.findUserByUsername(username);
    }

    @Override
    public UserDetails loadUserByUsername(String s) throws UsernameNotFoundException {
        User user = findUserByUsername(s);
        if (user == null) {
            throw new BadCredentialsException("用户不存在");
        } else {
            if (!user.isAccountNonExpired()) {
                throw new BadCredentialsException("账户已过期");
            } else if (!user.isAccountNonLocked()) {
                throw new LockedException("账户被锁定");
            } else if (!user.isCredentialsNonExpired()) {
                throw new CredentialsExpiredException("凭据已过期");
            } else if (!user.isEnabled()) {
                throw new DisabledException("账户被禁用");
            } else {
                List<GrantedAuthority> grantedAuthorityList = new ArrayList<>();
                Set<UserRole> userRoleSet = user.getUserRoleSet();
                for (UserRole userRole : userRoleSet) {
                    grantedAuthorityList.add(new SimpleGrantedAuthority("ROLE_" + userRole.getRole().name()));
                }
                new org.springframework.security.core.userdetails.User(user.getUsername(), user.getPassword(), grantedAuthorityList);
            }
        }
        return user;
    }
}
