package cn.minbb.edu.task;

import cn.minbb.edu.model.UserRole;
import cn.minbb.edu.service.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

@Order(1)
@Component
public class InitDataRunner implements ApplicationRunner {

    private Logger logger = LoggerFactory.getLogger(InitDataRunner.class);

    private UserService userService;
    private UserRoleService userRoleService;
    private CourseService courseService;
    private BannerService bannerService;
    private SubjectService subjectService;

    public InitDataRunner(UserService userService, UserRoleService userRoleService, CourseService courseService, BannerService bannerService, SubjectService subjectService) {
        this.userService = userService;
        this.userRoleService = userRoleService;
        this.courseService = courseService;
        this.bannerService = bannerService;
        this.subjectService = subjectService;
    }

    @Override
    public void run(ApplicationArguments args) throws Exception {
        // 检查用户角色 - 初始化用户角色数据
        for (UserRole.Role role : UserRole.Role.values()) {
            if (null == userRoleService.findOneByRole(role)) {
                logger.info("初始化用户角色数据 -> {} = {}", role.name(), role.getDescription());
                userRoleService.saveOne(new UserRole(role, role.getDescription()));
            }
        }
    }
}
